{ **************************************************************************** }
{ *                                                                          * }
{ * SNOOPY (Nintendo, Game & Watch Panorama Screen) Simulator                * }
{ * by Luca "MADrigal" Antignano, October 2000 (revision: October 2022)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit gfxinit;
{ this unit contains the graphics initialization (skin-based system)           }

interface

uses
  Forms, Controls, StdCtrls, SysUtils, Classes, Graphics, ExtCtrls, Dialogs,
  Jpeg, Pngimage, IniFiles;

{ * PROCEDURES: SKIN INITIALIZATION                                          * }
procedure Gfx_Game_Init;
procedure Gfx_Manual_Init;

const
  INIFILE = 'Config.ini';

{}IMAGES = 128;

{}DEFAULTHEIGHT    = 770;
{}DEFAULTWIDTH     = 498;
  DEFAULTSKINDIR   = 'Graphics\Skin\';
  DEFAULTWINMODE   = 'Shaped';

  DEFAULTTRCOLOR   = '$00ff00ff';

  DEFAULTBDDIR     = 'Graphics\Backdrop\';
  DEFAULTBDFILE    = 'nil';
  DEFAULTBDSTRETCH = 0;
{}DEFAULTBGCOLOR   = '$00000000';
  DEFAULTOFFSETX   = 'Center';
  DEFAULTOFFSETY   = 'Center';
  DEFAULTLOGOFILE  = 'nil';
  DEFAULTLOGOPOS   = 1;

  DEFAULTMANUALDIR = 'Graphics\Manual\';
{}DEFAULTMANHEIGHT = 365;
{}DEFAULTMANWIDTH  = 545;

var
  IF_File: TIniFile;
  aImages: array [1..IMAGES] of TImage;
{}//sVideoMode: string;

implementation

uses
  manual, splash, unit1;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: SKIN INITIALIZATION                                          * }
{ *                                                                          * }
{ **************************************************************************** }

procedure Gfx_Game_Init;
{ initializes the graphics on the main game form                               }
  var
    // service variables
    i,
    iOffsetX,
    iOffsetY: integer;

    // temporary variables (used to read/write from 'ini file')
    sSkinDir,
    sBackdropDir,
    sBackdropFile,
    sGameOffsetX,
    sGameOffsetY,
{}  sLogoFile: string;
{}//sWindowMode

    // image variables
    sFilename: string;
begin
  // initialises array of images
  with Form1 do
    begin
      aImages[1]  := im_background_bottom;       aImages[2]  := im_background_back;
      aImages[3]  := im_background_front;
      aImages[4]  := im_background_screen_top;

      aImages[5]  := btn_open;
      aImages[6]  := btn_game_a_down;            aImages[7]  := btn_game_a_top;
      aImages[8]  := btn_game_b_down;            aImages[9]  := btn_game_b_top;
      aImages[10] := btn_time_down;              aImages[11] := btn_time_top;
      aImages[12] := btn_acl_down;               aImages[13] := btn_acl_top;
                                                 aImages[14] := btn_alarm_top;

      aImages[15] := btn_move_down_left;         aImages[16] := btn_move_down_right;
                                                 aImages[17] := btn_move_top;

      aImages[18] := btn_hit_down;               aImages[19] := btn_hit_top;

      aImages[20] := btn_ctrlr_overlay_left;
      aImages[21] := btn_ctrlr_overlay_right;

      aImages[22] := im_game_background;

      aImages[23] := im_panel_miss_0; aImages[24] := im_panel_miss_1; aImages[25] := im_panel_miss_2; aImages[26] := im_panel_miss_3;
      aImages[27] := im_panel_alarm_1; aImages[28] := im_panel_alarm_2; aImages[29] := im_panel_alarm_3;
      aImages[30] := im_panel_game_a; aImages[31] := im_panel_game_b;
      aImages[32] := im_panel_time_am; aImages[33] := im_panel_time_pm; aImages[34] := im_panel_time_colon;
      aImages[35] := im_panel_digit_1; aImages[36] := im_panel_digit_2; aImages[37] := im_panel_digit_3; aImages[38] := im_panel_digit_4;
      aImages[39] := im_panel_number_0; aImages[40] := im_panel_number_1; aImages[41] := im_panel_number_2; aImages[42] := im_panel_number_3; aImages[43] := im_panel_number_4;
      aImages[44] := im_panel_number_5; aImages[45] := im_panel_number_6; aImages[46] := im_panel_number_7; aImages[47] := im_panel_number_8; aImages[48] := im_panel_number_9;

      aImages[49] := im_lucy_1; aImages[50] := im_lucy_2; aImages[51] := im_lucy_3;
      aImages[52] := im_lucy_1_snore_1; aImages[53] := im_lucy_3_crash;
      aImages[54] := im_lucy_1_snore_2;

      aImages[55] := im_schroeder_1; aImages[56] := im_schroeder_2;
      aImages[57] := im_schroeder_1_hand_1;
      aImages[58] := im_schroeder_1_hand_2;
      aImages[59] := im_schroeder_1_hand_3;

      aImages[60] := im_piano_1; aImages[61] := im_piano_2;

      aImages[62] := im_woodstock_1_up;      aImages[63] := im_woodstock_2_up;      aImages[64] := im_woodstock_3_up;      aImages[65] := im_woodstock_4_up;
      aImages[66] := im_woodstock_1_down;    aImages[67] := im_woodstock_2_down;    aImages[68] := im_woodstock_3_down;    aImages[69] := im_woodstock_4_down;
      aImages[70] := im_woodstock_1_snore_1; aImages[71] := im_woodstock_2_snore_1; aImages[72] := im_woodstock_3_snore_1; aImages[73] := im_woodstock_4_snore_1;
      aImages[74] := im_woodstock_1_snore_2; aImages[75] := im_woodstock_2_snore_2; aImages[76] := im_woodstock_3_snore_2; aImages[77] := im_woodstock_4_snore_2;
      aImages[78] := im_woodstock_1_snore_3; aImages[79] := im_woodstock_2_snore_3; aImages[80] := im_woodstock_3_snore_3; aImages[81] := im_woodstock_4_snore_3;

      aImages[82] := im_snoopy_0_down;        aImages[83] := im_snoopy_1_down;        aImages[84] := im_snoopy_2_down;        aImages[85] := im_snoopy_3_down;        aImages[86] := im_snoopy_4_down; aImages[87] := im_snoopy_5_down;
      aImages[88] := im_snoopy_0_up;          aImages[89] := im_snoopy_1_up;          aImages[90] := im_snoopy_2_up;          aImages[91] := im_snoopy_3_up;          aImages[92] := im_snoopy_4_up;
      aImages[93] := im_snoopy_1_hammer_down; aImages[94] := im_snoopy_2_hammer_down; aImages[95] := im_snoopy_3_hammer_down; aImages[96] := im_snoopy_4_hammer_down;
      aImages[97] := im_snoopy_1_hammer_up;   aImages[98] := im_snoopy_2_hammer_up;   aImages[99] := im_snoopy_3_hammer_up;   aImages[100] := im_snoopy_4_hammer_up;

      aImages[101] := im_note_1_5;       aImages[102] := im_note_2_5;       aImages[103] := im_note_3_5;       aImages[104] := im_note_4_5;
      aImages[105] := im_note_1_4;       aImages[106] := im_note_2_4;       aImages[107] := im_note_3_4;       aImages[108] := im_note_4_4;
      aImages[109] := im_note_1_3;       aImages[110] := im_note_2_3;       aImages[111] := im_note_3_3;       aImages[112] := im_note_4_3;
      aImages[113] := im_note_1_2;       aImages[114] := im_note_2_2;       aImages[115] := im_note_3_2;       aImages[116] := im_note_4_2;
      aImages[117] := im_note_1_1;       aImages[118] := im_note_2_1;       aImages[119] := im_note_3_1;       aImages[120] := im_note_4_1;
      aImages[121] := im_note_1_4_hit_1; aImages[122] := im_note_2_4_hit_1; aImages[123] := im_note_3_4_hit_1; aImages[124] := im_note_4_4_hit_1;
      aImages[125] := im_note_1_4_hit_2; aImages[126] := im_note_2_4_hit_2; aImages[127] := im_note_3_4_hit_2; aImages[128] := im_note_4_4_hit_2;
    end;

  // TO BE USED TO GENERATE THE SKIN FILE ONLY (TO BE COMMENTED THEREAFTER)
{  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'Export.ini');

  for i := 1 to IMAGES do
    begin
      if aImages[i].Picture.Graphic = nil then
        sFilename := 'nil'
      else
        if aImages[i].Picture.Graphic is TJPEGImage then
          sFilename := aImages[i].Name + '.jpg'
        else
          if aImages[i].Picture.Graphic is TBitmap then
            sFilename := aImages[i].Name + '.bmp'
          else
            if aImages[i].Picture.Graphic is TPNGObject then
              sFilename := aImages[i].Name + '.png';

      IF_file.WriteString('Image' + IntToStr(i),  'Filename', sFilename);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Height',   aImages[i].Height);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Left',     aImages[i].Left);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Top',      aImages[i].Top);
      IF_file.WriteInteger('Image' + IntToStr(i), 'Width',    aImages[i].Width);
    end;

  IF_File.Destroy; }

  // loads configuration settings from the Config.ini file
  if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
    begin
      MessageDlg('Configuration file (Config.ini) not found. Please reinstall the game.', mtError, [mbOK], 0);
      Application.Terminate;
    end
  else
    begin
      // reads init values from 'ini file' -- with defaults if no 'ini file' found
      IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

      // game form initialization (game graphics and backdrop)
      with Form1 do
        begin
          sSkinDir     := IncludeTrailingPathDelimiter(IF_file.ReadString('Common', 'SkinDir', DEFAULTSKINDIR));
          ClientHeight := IF_file.ReadInteger('Common', 'GameHeight', DEFAULTHEIGHT);
          ClientWidth  := IF_file.ReadInteger('Common', 'GameWidth',  DEFAULTWIDTH);

          // use Fullscreen or Shaped mode
{}        if bFullscreenMode then
            begin                                                               // Fullscreen
              // sets backdrop image and background colour
              sBackdropDir  := IncludeTrailingPathDelimiter(IF_file.ReadString('Fullscreen', 'BackdropDir',  DEFAULTBDDIR));
              sBackdropFile := IF_file.ReadString('Fullscreen', 'BackdropFile', DEFAULTBDFILE);

              if (sBackdropFile <> '')
                and (sBackdropFile <> 'nil') then
                if FileExists(sBackdropDir + sBackdropFile) then
                  begin
                    im_backdrop.Picture.LoadFromFile(sBackdropDir + sBackdropFile);
                    im_backdrop.Stretch := Boolean(IF_file.ReadInteger('Fullscreen', 'BackdropStretch', DEFAULTBDSTRETCH));
{}                  im_backdrop.Visible := true;
                  end;

              Color := StrToInt(IF_file.ReadString('Fullscreen', 'BgColor', DEFAULTBGCOLOR));
              TransparentColor := false;

              // sets new coordinates for the handheld game graphics
              sGameOffsetX := IF_file.ReadString('Fullscreen', 'GameOffsetX', DEFAULTOFFSETX);
              sGameOffsetY := IF_file.ReadString('Fullscreen', 'GameOffsetY', DEFAULTOFFSETY);

              if StrToIntDef(sGameOffsetX, 0) <> 0 then iOffsetX := StrToInt(sGameOffsetX) else iOffsetX := Trunc((Screen.Width  - ClientWidth)  / 2);
              if StrToIntDef(sGameOffsetY, 0) <> 0 then iOffsetY := StrToInt(sGameOffsetY) else iOffsetY := Trunc((Screen.Height - ClientHeight) / 2);

              // sets the form as full screen
              ClientWidth  := Screen.Width;
              ClientHeight := Screen.Height;

              // sets logo (watermark) image and its position
              sLogoFile := IF_file.ReadString('Fullscreen', 'LogoFile', DEFAULTLOGOFILE);

              if (sLogoFile <> '')
                and (sLogoFile <> 'nil') then
                if FileExists(sBackdropDir + sLogoFile) then
                  with Form1 do
                    begin
                      im_backdrop_logo.Picture.LoadFromFile(sBackdropDir + sLogoFile);
                      im_backdrop_logo.AutoSize := true;
{}                    im_backdrop_logo.Visible := true;

                      case IF_file.ReadInteger('Fullscreen', 'LogoPosition', DEFAULTLOGOPOS) of
                        0: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := 0; end; // top-left
                        1: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := 0; end; // top-right
                        2: begin im_backdrop_logo.Left := 0;                                    im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-left
                        3: begin im_backdrop_logo.Left := ClientWidth - im_backdrop_logo.Width; im_backdrop_logo.Top := ClientHeight - im_backdrop_logo.Height; end; // bottom-right
                      end; // case..of
                    end;
            end
          else                                                                  // Shaped
            begin
              // sets coordinates for the handheld game graphics
              iOffsetX := 0;
              iOffsetY := 0;

              // sets the form as 'shaped'
              Color                 := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColorValue := StrToInt(IF_file.ReadString('VideoMode', 'TranspColor', DEFAULTTRCOLOR));
              TransparentColor      := true;

              // disables the backdrop
              im_backdrop.Visible := false;
              im_backdrop_logo.Visible := false;
            end;
        end;

      // game skin initialization
      for i := 1 to IMAGES do
        begin
          sFilename := IF_file.ReadString('Image' + IntToStr(i), 'Filename', '');

          if (sFilename <> '')
            and (sFilename <> 'nil') then
            if FileExists(sSkinDir + sFilename) then
              // loads picture into the image
              aImages[i].Picture.LoadFromFile(sSkinDir + sFilename)
            else
              // image expected but not found -> creates a coloured square as a placeholder
              begin
                aImages[i].Picture.Bitmap.Create;
                aImages[i].Picture.Bitmap.Width  := 10;
                aImages[i].Picture.Bitmap.Height := 10;
                aImages[i].Picture.Bitmap.Canvas.Create;
                aImages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
                aImages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
                aImages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
                aImages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
              end;

          aImages[i].Height := IF_file.ReadInteger('Image' + IntToStr(i), 'Height', 0);
          aImages[i].Left   := IF_file.ReadInteger('Image' + IntToStr(i), 'Left',   0) + iOffsetX;
          aImages[i].Top    := IF_file.ReadInteger('Image' + IntToStr(i), 'Top',    0) + iOffsetY;
          aImages[i].Width  := IF_file.ReadInteger('Image' + IntToStr(i), 'Width',  0);
        end;

      // close Inifile
      IF_File.Destroy;
    end;

  // initializes the on-screen game menu
  with Form1 do
    begin
{}    btn_game_menu_top.Visible := bFullscreenMode;
{}    btn_game_menu_background.Visible := bFullscreenMode;

      btn_game_menu_background.Height := 32;
      btn_game_menu_background.Width := 36;
    end;
end;





procedure Gfx_Manual_Init;
{ initializes the graphics of the instructions manual form                     }
  var
    // service variables
    i: integer;

    // temporary variables (used to read/write from 'ini file')
    sManualDir: string;

    // image variables
    sFilename: string;
begin
  // reads init values from 'ini file' -- with defaults if no 'ini file' found
  IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);

  // manual form initialization (graphics and sizes)
  with ManualForm do
    begin
      sManualDir   := IncludeTrailingPathDelimiter(IF_file.ReadString('Manual', 'ManualDir', DEFAULTMANUALDIR));

      ClientHeight := IF_file.ReadInteger('Manual', 'ManualHeight', DEFAULTMANHEIGHT);
      ClientWidth  := IF_file.ReadInteger('Manual', 'ManualWidth',  DEFAULTMANWIDTH);

      im_page_top.Left   := 0;
      im_page_top.Top    := 0;
      im_page_top.Height := ClientHeight;
      im_page_top.Width  := ClientWidth;

      sh_page_top.Left   := 0;
      sh_page_top.Top    := 0;
      sh_page_top.Height := ClientHeight;
      sh_page_top.Width  := ClientWidth;
    end;

  // instructions manual skin initialization
  for i := 1 to MANUALPAGES do
    begin
      sFilename := IntToStr(i) + '.bmp';

      if FileExists(sManualDir + sFilename) then
          // loads picture into the image
          aIm_pages[i].Picture.LoadFromFile(sManualDir + sFilename)
        else
          // image expected but not found -> creates a coloured square as a placeholder
          begin
            aIm_pages[i].Picture.Bitmap.Create;
            aIm_pages[i].Picture.Bitmap.Width  := 10;
            aIm_pages[i].Picture.Bitmap.Height := 10;
            aIm_pages[i].Picture.Bitmap.Canvas.Create;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Color := 0;
            aIm_pages[i].Picture.Bitmap.Canvas.Pen.Style := psSolid;
            aIm_pages[i].Picture.Bitmap.Canvas.Brush.Color := $000000ff;
            aIm_pages[i].Picture.Bitmap.Canvas.Rectangle(0, 0, 10, 10);
          end;

      with ManualForm do
        begin
          aIm_pages[i].Height := ClientHeight;
          aIm_pages[i].Width  := ClientWidth;
        end;
    end;

  // close Inifile
  IF_File.Destroy;
end;





end.
