{ **************************************************************************** }
{ *                                                                          * }
{ * SNOOPY (Nintendo, Game & Watch Panorama Screen) Simulator                * }
{ * by Luca "MADrigal" Antignano, October 2000 (revision: October 2022)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:          TImage;
    btn_game_menu_background:   TImage;

    im_backdrop:                TImage;
    im_backdrop_logo:           TImage;

    // electronic game
    im_background_bottom:       TImage;     im_background_front:      TImage;
                                            im_background_back:       TImage;

                                            im_background_screen_top: TImage;

    btn_open:                   TImage;

    btn_move_down_left:         TImage;     btn_move_down_right:      TImage;
                                            btn_move_top:             TImage;

    btn_hit_down:               TImage;     btn_hit_top:              TImage;

    btn_game_a_down:            TImage;     btn_game_a_top:           TImage;
    btn_game_b_down:            TImage;     btn_game_b_top:           TImage;
    btn_time_down:              TImage;     btn_time_top:             TImage;

    btn_acl_down:               TImage;     btn_acl_top:              TImage;
                                            btn_alarm_top:            TImage;

    btn_ctrlr_overlay_left:     TImage;
    btn_ctrlr_overlay_right:    TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (background, alarm, time/score, game a/b, etc.)
    im_game_background: TImage;

    im_panel_miss_0: TImage; im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage;

    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_digit_1: TImage; im_panel_digit_2: TImage; im_panel_digit_3: TImage; im_panel_digit_4: TImage;
    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_lucy_1: TImage;         im_lucy_2: TImage; im_lucy_3: TImage;
    im_lucy_1_snore_1: TImage; {                } im_lucy_3_crash: TImage;
    im_lucy_1_snore_2: TImage; {                } {                      }

    im_schroeder_1: TImage;        im_schroeder_2: TImage;
    im_schroeder_1_hand_1: TImage; {                     }
    im_schroeder_1_hand_2: TImage; {                     }
    im_schroeder_1_hand_3: TImage; {                     }

    im_piano_1: TImage; im_piano_2: TImage;

    {                       } im_woodstock_1_up: TImage;       im_woodstock_2_up: TImage;       im_woodstock_3_up: TImage;       im_woodstock_4_up: TImage;       {                       }
    {                       } im_woodstock_1_down: TImage;     im_woodstock_2_down: TImage;     im_woodstock_3_down: TImage;     im_woodstock_4_down: TImage;     {                       }
    {                       } im_woodstock_1_snore_1: TImage;  im_woodstock_2_snore_1: TImage;  im_woodstock_3_snore_1: TImage;  im_woodstock_4_snore_1: TImage;  {                       }
    {                       } im_woodstock_1_snore_2: TImage;  im_woodstock_2_snore_2: TImage;  im_woodstock_3_snore_2: TImage;  im_woodstock_4_snore_2: TImage;  {                       }
    {                       } im_woodstock_1_snore_3: TImage;  im_woodstock_2_snore_3: TImage;  im_woodstock_3_snore_3: TImage;  im_woodstock_4_snore_3: TImage;  {                       }

    im_snoopy_0_down: TImage; im_snoopy_1_down: TImage;        im_snoopy_2_down: TImage;        im_snoopy_3_down: TImage;        im_snoopy_4_down: TImage;        im_snoopy_5_down: TImage;
    im_snoopy_0_up: TImage;   im_snoopy_1_up: TImage;          im_snoopy_2_up: TImage;          im_snoopy_3_up: TImage;          im_snoopy_4_up: TImage;          {                       }
    {                       } im_snoopy_1_hammer_down: TImage; im_snoopy_2_hammer_down: TImage; im_snoopy_3_hammer_down: TImage; im_snoopy_4_hammer_down: TImage; {                       }
    {                       } im_snoopy_1_hammer_up: TImage;   im_snoopy_2_hammer_up: TImage;   im_snoopy_3_hammer_up: TImage;   im_snoopy_4_hammer_up: TImage;   {                       }

    {                       } im_note_1_5: TImage;             im_note_2_5: TImage;             im_note_3_5: TImage;             im_note_4_5: TImage;             {                       }
    {                       } im_note_1_4: TImage;             im_note_2_4: TImage;             im_note_3_4: TImage;             im_note_4_4: TImage;             {                       }
    {                       } im_note_1_3: TImage;             im_note_2_3: TImage;             im_note_3_3: TImage;             im_note_4_3: TImage;             {                       }
    {                       } im_note_1_2: TImage;             im_note_2_2: TImage;             im_note_3_2: TImage;             im_note_4_2: TImage;             {                       }
    {                       } im_note_1_1: TImage;             im_note_2_1: TImage;             im_note_3_1: TImage;             im_note_4_1: TImage;             {                       }
    {                       } im_note_1_4_hit_1: TImage;       im_note_2_4_hit_1: TImage;       im_note_3_4_hit_1: TImage;       im_note_4_4_hit_1: TImage;       {                       }
    {                       } im_note_1_4_hit_2: TImage;       im_note_2_4_hit_2: TImage;       im_note_3_4_hit_2: TImage;       im_note_4_4_hit_2: TImage;

{ * TIMERS                                                                   * }
    Timer_Clock:                  TTimer;
    Timer_Game:                   TTimer;
    Timer_Game_Over:              TTimer;
    Timer_Game_Start:             TTimer;
    Timer_Intermission:           TTimer;
    Timer_Intermission_No_Chance: TTimer;
    Timer_Miss_Fall:              TTimer;
    Timer_Miss_Note_Snoopy:       TTimer;
    Timer_Miss_Note_Woodstock:    TTimer;
    Timer_Score_Blink:            TTimer;
    Timer_Snoopy_Hammer:          TTimer;
    Timer_Snoopy_Hit_Note:        TTimer;
    Timer_Snoopy_Jump:            TTimer;
    Timer_Woodstock:              TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Hero             (b: boolean);
    procedure Vid_Show_All_Notes            (b: boolean);
    procedure Vid_Show_All_Lucy             (b: boolean);
    procedure Vid_Show_All_Piano            (b: boolean);
    procedure Vid_Show_All_Schroeder        (b: boolean);
    procedure Vid_Show_All_Woodstock        (b: boolean);

    procedure Vid_Show_Panel_Miss           (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

    procedure Vid_Show_Intermission_Frame   (p: integer);

    procedure Vid_Show_Notes_Row            (p: integer; b: boolean);
    procedure Vid_Show_Snoopy               (p, q: integer; b: boolean);
    procedure Vid_Show_Woodstock_Row        (p: integer; b: boolean);
    procedure Vid_Show_Woodstock_Snore_Column(p: integer; b: boolean);
    procedure Vid_Show_Woodstock_Snore_Row  (p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_background_bottomMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_background_bottomMouseMove (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_openMouseDown             (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_hit_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_hit_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_leftMouseDown(Sender: TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_open_down;

    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;
    procedure Inp_hit_down;                 procedure Inp_hit_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Intermission;
    procedure Gam_Execute_Miss;
    procedure Gam_Init_Stage;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_IntermissionTimer       (Sender: TObject);
    procedure Timer_Intermission_No_ChanceTimer(Sender: TObject);
    procedure Timer_Miss_FallTimer          (Sender: TObject);
    procedure Timer_Miss_Note_SnoopyTimer   (Sender: TObject);
    procedure Timer_Miss_Note_WoodstockTimer(Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);
    procedure Timer_Snoopy_HammerTimer      (Sender: TObject);
    procedure Timer_Snoopy_Hit_NoteTimer    (Sender: TObject);
    procedure Timer_Snoopy_JumpTimer        (Sender: TObject);
    procedure Timer_WoodstockTimer          (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Snoopy (Nintendo, Game & Watch Panorama Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.01';
  HH2_CONTENT_DATE    = '2022/10/17';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Graphic improvements *'; // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 17th October, 2022'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\SnoopyP50';                                  // registry key

  HINT_CLICK  = 'click or press';
  HINT_OPEN   = 'to open the game';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';
  HINT_MOVE   = 'activate the controller overlays or use';

{}MENU_ITEMS = 14;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_chance,
  pfs_gameover,
  pfs_hammer1,
  pfs_hammer2,
  pfs_interm1,
  pfs_interm2,
  pfs_jump,
  pfs_miss1,
  pfs_miss2,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of sound samples * }
  aPFnote:                 array[1..28] of PFSoundSample;

{ * arrays of images * }
  aIm_panel_miss:          array[0..3]        of TImage;                        // miss marks panel
  aIm_panel_alarm:         array[1..3]        of TImage;                        // alarm panel
  aIm_panel_digit_pos:     array[1..4]        of TImage;                        // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]        of TImage;                        // digits for score/time

  aIm_notes:               array[1..4, 1..5]  of TImage;
  aIm_notes_hit:           array[1..4, 1..2]  of TImage;                        // used when a note has been hit
  aIm_schroeder_hands:     array[1..3]        of TImage;

  aIm_snoopy:              array[0..5]        of record
                                                   down, up,
                                                   hammer_up, hammer_down: TImage;
                                                 end;

  aIm_woodstock:           array[1..4]        of record
                                                   down, up: TImage;
                                                   snore: array[1..3] of TImage;
                                                 end;

{ * arrays of integers * }
  aDifficulty:             array[0..19, 1..7] of integer;                       // first index: hundreds in score; second index: units in score

{ * arrays of mixed types *}
  aKeys:                   array[1..9]       of record                          // keys management (values)
                                                  int:  integer;
                                                  word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=closed, 1/2=ACL 1/2, 3=alarm, 4=time, 5/7=hiscore A/B, 6/8=game A/B, 9=game over
  iScore,                                                                       // score=0-1999 (then resets to 0)
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999
  iScoreMultiplier,                                                             // given points=1 (normal) or 2 (chance-time)
  iScoreNextIntermission,                                                       // at every 100 points
  iScoreNextChance,                                                             // 300 and every further 1000 points

  iPosHeroX,                                                                    // X coordinate of hero
  iMiss,                                                                        // number of misses: 0-3
  iMaxRandom,                                                                   // lower number = higher chances that a note is generated

  iNoteBaseTone,                                                                // base tone for playing notes (increases as game proceeds)
  iNoteDisabledPosX                                                             // X coordinate of disabled column of notes (in Game A)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bControls,
  bControlsHammer,
  bSnoopyJumpsLeft,

  bChance
                         : boolean;

  // Game variables: loops
  iLoopGame,
  iLoopGameAnimation,
  iLoopGameOver,
  iLoopGameOverBlink, // up to 425 blinks of 700 msec. each
  iLoopGameStart,
  iLoopIntermission,
  iLoopIntermissionNoChance,
  iloopMissFall,
  iLoopMissNoteSnoopy,
  iLoopMissNoteWoodstock,
  iLoopMissNoteWoodstockPosX,
  iLoopScoreBlink,
  iLoopSnoopyHitNote,
  iLoopSnoopyHitNotePosX,
  iLoopWoodstock
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i, j: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_miss[0] := im_panel_miss_0; aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3;
  aIm_panel_digit_pos[1] := im_panel_digit_1; aIm_panel_digit_pos[2] := im_panel_digit_2; aIm_panel_digit_pos[3] := im_panel_digit_3; aIm_panel_digit_pos[4] := im_panel_digit_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  {                                     } aIm_woodstock[1].down := im_woodstock_1_down;         aIm_woodstock[2].down := im_woodstock_2_down;         aIm_woodstock[3].down := im_woodstock_3_down;         aIm_woodstock[4].down := im_woodstock_4_down;         {                                     }
  {                                     } aIm_woodstock[1].up := im_woodstock_1_up;             aIm_woodstock[2].up := im_woodstock_2_up;             aIm_woodstock[3].up := im_woodstock_3_up;             aIm_woodstock[4].up := im_woodstock_4_up;             {                                     }

  {                                     } aIm_woodstock[1].snore[1] := im_woodstock_1_snore_1;  aIm_woodstock[2].snore[1] := im_woodstock_2_snore_1;  aIm_woodstock[3].snore[1] := im_woodstock_3_snore_1;  aIm_woodstock[4].snore[1] := im_woodstock_4_snore_1;  {                                     }
  {                                     } aIm_woodstock[1].snore[2] := im_woodstock_1_snore_2;  aIm_woodstock[2].snore[2] := im_woodstock_2_snore_2;  aIm_woodstock[3].snore[2] := im_woodstock_3_snore_2;  aIm_woodstock[4].snore[2] := im_woodstock_4_snore_2;  {                                     }
  {                                     } aIm_woodstock[1].snore[3] := im_woodstock_1_snore_3;  aIm_woodstock[2].snore[3] := im_woodstock_2_snore_3;  aIm_woodstock[3].snore[3] := im_woodstock_3_snore_3;  aIm_woodstock[4].snore[3] := im_woodstock_4_snore_3;  {                                     }

  aIm_snoopy[0].down := im_snoopy_0_down; aIm_snoopy[1].down := im_snoopy_1_down;               aIm_snoopy[2].down := im_snoopy_2_down;               aIm_snoopy[3].down := im_snoopy_3_down;               aIm_snoopy[4].down := im_snoopy_4_down;               aIm_snoopy[5].down := im_snoopy_5_down;
  aIm_snoopy[0].up := im_snoopy_0_up;     aIm_snoopy[1].up := im_snoopy_1_up;                   aIm_snoopy[2].up := im_snoopy_2_up;                   aIm_snoopy[3].up := im_snoopy_3_up;                   aIm_snoopy[4].up := im_snoopy_4_up;                   {                                     }
  {                                     } aIm_snoopy[1].hammer_up := im_snoopy_1_hammer_up;     aIm_snoopy[2].hammer_up := im_snoopy_2_hammer_up;     aIm_snoopy[3].hammer_up := im_snoopy_3_hammer_up;     aIm_snoopy[4].hammer_up := im_snoopy_4_hammer_up;     {                                     }
  {                                     } aIm_snoopy[1].hammer_down := im_snoopy_1_hammer_down; aIm_snoopy[2].hammer_down := im_snoopy_2_hammer_down; aIm_snoopy[3].hammer_down := im_snoopy_3_hammer_down; aIm_snoopy[4].hammer_down := im_snoopy_4_hammer_down; {                                     }

  {                                     } aIm_notes[1, 5] := im_note_1_5;                       aIm_notes[2, 5] := im_note_2_5;                       aIm_notes[3, 5] := im_note_3_5;                       aIm_notes[4, 5] := im_note_4_5;                       {                                     }
  {                                     } aIm_notes[1, 4] := im_note_1_4;                       aIm_notes[2, 4] := im_note_2_4;                       aIm_notes[3, 4] := im_note_3_4;                       aIm_notes[4, 4] := im_note_4_4;                       {                                     }
  {                                     } aIm_notes[1, 3] := im_note_1_3;                       aIm_notes[2, 3] := im_note_2_3;                       aIm_notes[3, 3] := im_note_3_3;                       aIm_notes[4, 3] := im_note_4_3;                       {                                     }
  {                                     } aIm_notes[1, 2] := im_note_1_2;                       aIm_notes[2, 2] := im_note_2_2;                       aIm_notes[3, 2] := im_note_3_2;                       aIm_notes[4, 2] := im_note_4_2;                       {                                     }
  {                                     } aIm_notes[1, 1] := im_note_1_1;                       aIm_notes[2, 1] := im_note_2_1;                       aIm_notes[3, 1] := im_note_3_1;                       aIm_notes[4, 1] := im_note_4_1;                       {                                     }

  {                                     } aIm_notes_hit[1, 1] := im_note_1_4_hit_1;             aIm_notes_hit[2, 1] := im_note_2_4_hit_1;             aIm_notes_hit[3, 1] := im_note_3_4_hit_1;             aIm_notes_hit[4, 1] := im_note_4_4_hit_1;             {                                     }
  {                                     } aIm_notes_hit[1, 2] := im_note_1_4_hit_2;             aIm_notes_hit[2, 2] := im_note_2_4_hit_2;             aIm_notes_hit[3, 2] := im_note_3_4_hit_2;             aIm_notes_hit[4, 2] := im_note_4_4_hit_2;             {                                     }

  aIm_schroeder_hands[1] := im_schroeder_1_hand_1; aIm_schroeder_hands[2] := im_schroeder_1_hand_2; aIm_schroeder_hands[3] := im_schroeder_1_hand_3;

  // arrays of integers (difficulty settings)
  aDifficulty[0, 1] := 75; aDifficulty[0, 2] := 70; aDifficulty[0, 3] := 70; aDifficulty[0, 4] := 65; aDifficulty[0, 5] := 65; aDifficulty[0, 6] := 60; aDifficulty[0, 7] := 50;
  aDifficulty[1, 1] := 75; aDifficulty[1, 2] := 70; aDifficulty[1, 3] := 70; aDifficulty[1, 4] := 65; aDifficulty[1, 5] := 60; aDifficulty[1, 6] := 55; aDifficulty[1, 7] := 50;
  aDifficulty[2, 1] := 75; aDifficulty[2, 2] := 70; aDifficulty[2, 3] := 65; aDifficulty[2, 4] := 60; aDifficulty[2, 5] := 60; aDifficulty[2, 6] := 55; aDifficulty[2, 7] := 45;
  aDifficulty[3, 1] := 70; aDifficulty[3, 2] := 65; aDifficulty[3, 3] := 65; aDifficulty[3, 4] := 60; aDifficulty[3, 5] := 55; aDifficulty[3, 6] := 50; aDifficulty[3, 7] := 45;
  aDifficulty[4, 1] := 70; aDifficulty[4, 2] := 65; aDifficulty[4, 3] := 60; aDifficulty[4, 4] := 60; aDifficulty[4, 5] := 55; aDifficulty[4, 6] := 50; aDifficulty[4, 7] := 40;
  aDifficulty[5, 1] := 70; aDifficulty[5, 2] := 65; aDifficulty[5, 3] := 60; aDifficulty[5, 4] := 55; aDifficulty[5, 5] := 50; aDifficulty[5, 6] := 45; aDifficulty[5, 7] := 40;
  aDifficulty[6, 1] := 65; aDifficulty[6, 2] := 60; aDifficulty[6, 3] := 55; aDifficulty[6, 4] := 55; aDifficulty[6, 5] := 50; aDifficulty[6, 6] := 45; aDifficulty[6, 7] := 35;
  aDifficulty[7, 1] := 65; aDifficulty[7, 2] := 60; aDifficulty[7, 3] := 55; aDifficulty[7, 4] := 50; aDifficulty[7, 5] := 45; aDifficulty[7, 6] := 40; aDifficulty[7, 7] := 35;
  aDifficulty[8, 1] := 60; aDifficulty[8, 2] := 55; aDifficulty[8, 3] := 50; aDifficulty[8, 4] := 50; aDifficulty[8, 5] := 45; aDifficulty[8, 6] := 40; aDifficulty[8, 7] := 30;
  aDifficulty[9, 1] := 60; aDifficulty[9, 2] := 55; aDifficulty[9, 3] := 50; aDifficulty[9, 4] := 45; aDifficulty[9, 5] := 40; aDifficulty[9, 6] := 35; aDifficulty[9, 7] := 30;

  for i := 10 to 19 do                                                          // sets difficulty for levels 11-20 just as that of level 10
    for j := 1 to 7 do
      aDifficulty[i, j] := aDifficulty[9, j];

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"   (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"  (default: "RIGHT")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 42);                             // "Move"   (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 7 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_open.Hint                := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_OPEN;   // "Open"

  btn_game_a_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint             := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint           := HINT_ALARM;                                                     // "Alarm"

  btn_move_top.Hint            := HINT_MOVE  + ' ' + aKeyHints[aKeys[5].int] + ', '               // "Left"
                                                   + aKeyHints[aKeys[6].int];                     // "Right"

  btn_hit_top.Hint             := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                     // "Hit"

  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bControlsHammer := false;                                                     // prevents hero movement

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
  var
    i: integer;
begin
  FSOUND_Sample_Free(pfs_chance);  FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_hammer1); FSOUND_Sample_Free(pfs_hammer2);
  FSOUND_Sample_Free(pfs_interm1); FSOUND_Sample_Free(pfs_interm2);
  FSOUND_Sample_Free(pfs_jump);    FSOUND_Sample_Free(pfs_miss1);
  FSOUND_Sample_Free(pfs_miss2);   FSOUND_Sample_Free(pfs_start);

  for i := 1 to 28 do
    FSOUND_Sample_Free(aPFnote[i]);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays;
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  for i := 1 to 3 do
    aIm_panel_alarm[i].Visible := b;

  im_game_background.Visible := b;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  Vid_Show_All_Hero(b);
  Vid_Show_All_Notes(b);
  Vid_Show_All_Lucy(b);
  Vid_Show_All_Piano(b);
  Vid_Show_All_Schroeder(b);
  Vid_Show_All_Woodstock(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all hero's sprites at once                                    }
  var
    i: integer;
begin
  for i := 0 to 5 do
    aIm_snoopy[i].down.Visible := b;

  for i := 0 to 4 do
    aIm_snoopy[i].up.Visible := b;

  for i := 1 to 4 do
    begin
      aIm_snoopy[i].hammer_up.Visible := b;
      aIm_snoopy[i].hammer_down.Visible := b;
    end;
end;





procedure TForm1.Vid_Show_All_Notes(b: boolean);
{ displays/hides all notes' sprites at once                                    }
  var
    i, j: integer;
begin
  for i := 1 to 4 do
    for j := 1 to 5 do
      aIm_notes[i, j].Visible := b;

  for i := 1 to 4 do
    for j := 1 to 2 do
      aIm_notes_hit[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Lucy(b: boolean);
{ displays/hides all Lucy's sprites at once                                    }
begin
  im_lucy_1.Visible := b;
  im_lucy_2.Visible := b;
  im_lucy_3.Visible := b;

  im_lucy_1_snore_1.Visible := b;
  im_lucy_1_snore_2.Visible := b;
  im_lucy_3_crash.Visible := b;
end;





procedure TForm1.Vid_Show_All_Piano(b: boolean);
{ displays/hides all the piano's sprites at once                               }
begin
  im_piano_1.Visible := b;
  im_piano_2.Visible := b;
end;





procedure TForm1.Vid_Show_All_Schroeder(b: boolean);
{ displays/hides all Schroeder's sprites at once                               }
  var
    i: integer;
begin
  im_schroeder_1.Visible := b;
  im_schroeder_2.Visible := b;

  for i := 1 to 3 do
    aIm_schroeder_hands[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Woodstock(b: boolean);
{ displays/hides all Woodstock's sprites at once                               }
  var
    i: integer;
begin
  for i := 1 to 4 do
    begin
      aIm_woodstock[i].down.Visible := b;
      aIm_woodstock[i].up.Visible := b;
    end;

  for i := 1 to 3 do
    Vid_Show_Woodstock_Snore_Row(i, b);
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  aIm_panel_miss[0].Visible := p > 0;                                           // displays/hides item 0

  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;                                            // thousands (always hidden)

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].Picture; // hundreds
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].Picture; // tens
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].Picture; // units
    end
  else                                                                            // hide
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Intermission_Frame(p: integer);
{ displays a specific frame of the Lucy/Schroeder intermission                 }
begin
  case p of
    1: begin
         Vid_Show_All_Lucy(false);

         im_lucy_2.Visible := true;
       end;
    2: begin
         im_lucy_2.Visible := false;

         im_lucy_3.Visible := true;
       end;
    3: im_lucy_3_crash.Visible := true;
    4: begin
         im_piano_1.Visible := false;

         im_piano_2.Visible := true;
       end;
    5: begin
         im_lucy_3_crash.Visible := false;
         Vid_Show_All_Schroeder(false);

         im_schroeder_2.Visible := true;
       end;
    6: begin
         im_piano_2.Visible := false;
         im_schroeder_2.Visible := false;
         im_lucy_3.Visible := false;
         aIm_schroeder_hands[3].Visible := false;

         aIm_schroeder_hands[1].Visible := true;
         aIm_schroeder_hands[2].Visible := true;
        end;
  end; // case..of
end;





procedure TForm1.Vid_Show_Notes_Row(p: integer; b: boolean);
{ displays/hides all the notes in row 'p'                                      }
  var
    i: integer;
begin
  for i :=1 to 4 do
    aIm_notes[i, p].Visible := b;
end;





procedure TForm1.Vid_Show_Snoopy(p, q: integer; b: boolean);
{ displays/hides Snoopy in column 'p' up (q = 1) or down (q = 2)               }
begin
  with aIm_snoopy[p] do
    case q of
      1: begin // up
           up.Visible := b;
         end;
      2: begin // down
           down.Visible := b;
           hammer_up.Visible := b;
           hammer_down.Visible := false;
         end;
    end; // case..of
end;





procedure TForm1.Vid_Show_Woodstock_Row(p: integer; b: boolean);
{ displays/hides all Woodstock's up (p = 1) or down (p = 2)                    }
  var
    i: integer;
begin
  for i := 1 to 4 do
    with aIm_woodstock[i] do
      case p of
        1: up.Visible := b;
        2: down.Visible := b;
      end;
end;





procedure TForm1.Vid_Show_Woodstock_Snore_Column(p: integer; b: boolean);
{ displays/hides all Woodstock's snores in column 'p'                          }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_woodstock[p].snore[i].Visible := b;
end;





procedure TForm1.Vid_Show_Woodstock_Snore_Row(p: integer; b: boolean);
{ displays/hides all Woodstock's snores in row 'p'                             }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_woodstock[i].snore[p].Visible := b;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PfSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_background_bottomMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_background_bottomMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_openMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Open button: PRESS                                                           }
begin
  if Button = mbLeft then
     Inp_open_down;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_hit_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ HIT button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_hit_down;
end;





procedure TForm1.btn_hit_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ HIT button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_hit_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then                                                   // Open or ACL
    if (iMode = 0) then
      Inp_open_down                                                               // Open
    else
      Inp_acl_down else                                                           // ACL
  if Key = aKeys[5].word then Inp_left_down   else                              // Left
  if Key = aKeys[6].word then Inp_right_down  else                              // Right

  if Key = aKeys[7].word then Inp_hit_down;                                     // Hit
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if (Key = aKeys[4].word) and (iMode > 0) then Inp_acl_up else                 // ACL

  if Key = aKeys[5].word then Inp_left_up   else                                // Left
  if Key = aKeys[6].word then Inp_right_up  else                                // Right

  if Key = aKeys[7].word then Inp_hit_up;                                       // Hit
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_open_down;
{ Open: PRESS                                                                  }
begin
  // background
  btn_open.Visible := false;
  im_background_front.Visible := false;
  im_background_back.Visible := true;
  im_background_screen_top.Visible := true;

  iMode := 2; Gam_Set_Mode;                                                     // closed -> ACL/2
end;





procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ Game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 5 then                                                             // hiscore A ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ Game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ Game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 7 then                                                             // hiscore B ->
    begin
      iMode := 8; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 3 then                                                             // alarm ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  if iMode <> 0 then                                                            // any mode except closed ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //                      -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 1 then                                                             // ACL/1 ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_left_down;
{ LEFT joystick: PRESS                                                         }
begin
  btn_move_top.Picture := btn_move_down_left.Picture;                           // button animation

  if bControls then
    begin
      bControls := false;

      Timer_Snoopy_Hammer.Enabled := false;

      Vid_Show_Snoopy(iPosHeroX, 2, false);

      bSnoopyJumpsLeft := true; // needed by both Timer_Miss_Fall and Timer_Snoopy_Jump timers

      if iPosHeroX = 1 then
        // Snoopy jumps off the leftmost platform: a miss occurs
        begin
          Gam_Stop_All_Timers;

          aIm_snoopy[0].up.Visible := true;

          iloopMissFall := 1;
          Timer_Miss_Fall.Interval := 100;
          Timer_Miss_Fall.Enabled := true;
        end
      else
        // Snoopy jumps left and lands onto a platform
        begin
          Vid_Show_Snoopy(iPosHeroX - 1, 1, true);

          Snd_Play(pfs_jump);

          Timer_Snoopy_Jump.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ LEFT joystick: RELEASE                                                       }
begin
  btn_move_top.Picture := nil;                                                  // button animation
end;





procedure TForm1.Inp_right_down;
{ RIGHT joystick: PRESS                                                        }
begin
  btn_move_top.Picture := btn_move_down_right.Picture;                          // button animation

  if bControls then
    begin
      bControls := false;

      Timer_Snoopy_Hammer.Enabled := false;

      Vid_Show_Snoopy(iPosHeroX, 2, false);

      bSnoopyJumpsLeft := false; // needed by both Timer_Miss_Fall and Timer_Snoopy_Jump timers

      if iPosHeroX = 4 then
        // Snoopy jumps off the rightmost platform: a miss occurs
        begin
          Gam_Stop_All_Timers;

          aIm_snoopy[4].up.Visible := true;

          iloopMissFall := 1;
          Timer_Miss_Fall.Interval := 100;
          Timer_Miss_Fall.Enabled := true;
        end
      else
        // Snoopy jumps right and lands onto a platform
        begin
          Vid_Show_Snoopy(iPosHeroX, 1, true);

          Snd_Play(pfs_jump);

          Timer_Snoopy_Jump.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ RIGHT joystick: RELEASE                                                      }
begin
  btn_move_top.Picture := nil;                                                  // button animation
end;





procedure TForm1.Inp_hit_down;
{ HIT button: PRESS                                                            }
begin
  btn_hit_down.Visible:=true;

  if bControls and bControlsHammer then
    begin
      bControlsHammer := false;

      aIm_snoopy[iPosHeroX].hammer_up.Visible := false;
      aIm_snoopy[iPosHeroX].hammer_down.Visible := true;

      if aIm_notes[iPosHeroX, 4].Visible then
        // Snoopy hits a note
        begin
          Snd_Play(pfs_hammer2);

          Timer_Snoopy_Hammer.Interval := 150;
          Timer_Snoopy_Hammer.Enabled := true;

          aIm_notes[iPosHeroX, 4].Visible := false;
          aIm_notes_hit[iPosHeroX, 1].Visible := true;

          iLoopSnoopyHitNote := 1;
          iLoopSnoopyHitNotePosX := iPosHeroX;
          Timer_Snoopy_Hit_Note.Interval := 1;
          Timer_Snoopy_Hit_Note.Enabled := false; // forces stop
          Timer_Snoopy_Hit_Note.Enabled := true;

          // increases score
          Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

          if iScore >= 2000 then
            begin
              Dec(iScore, 2000);
              iScoreNextIntermission := 0;
              iScoreNextChance := 300;
            end;

          Gam_Set_Difficulty;
          Gam_Check_Intermission;
        end
      else
        // Snoopy doesn't hit any note
        begin
          Snd_Play(pfs_hammer1);

          Timer_Snoopy_Hammer.Interval := 150;
          Timer_Snoopy_Hammer.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_hit_up;
{ HIT button: RELEASE                                                          }
begin
  bControlsHammer := true;
  btn_hit_down.Visible := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Clock.Enabled                  := false;
  Timer_Game.Enabled                   := false;
  Timer_Game_Over.Enabled              := false;
  Timer_Game_Start.Enabled             := false;
  Timer_Intermission.Enabled           := false;
  Timer_Intermission_No_Chance.Enabled := false;
  Timer_Miss_Fall.Enabled              := false;
  Timer_Miss_Note_Snoopy.Enabled       := false;
  Timer_Miss_Note_Woodstock.Enabled    := false;
  Timer_Score_Blink.Enabled            := false;
  Timer_Snoopy_Hammer.Enabled          := false;
  Timer_Snoopy_Hit_Note.Enabled        := false;
  Timer_Snoopy_Jump.Enabled            := false;
  Timer_Woodstock.Enabled              := false;
end;





procedure TForm1.Gam_Check_Intermission;
{ checks if an intermission must start (at every 100 points), including the    }
{ "chance-time" check                                                          }
  var
    iUnits,
    iHundreds: integer;
begin
  // calculates units and hundreds in score
  iUnits := iScore;
  iHundreds := 0;

  while iUnits > 99 do
    begin
      Dec(iUnits, 100);
      Inc(iHundreds);
    end;

  // intermission management
  if (iScore >= iScoreNextIntermission) then
    begin
      // the following timers are stopped one-by-one (no "Stop all timers) to
      // prevent the "blinking score" timer to stop, too
      Timer_Game.Enabled := false;
      Timer_Woodstock.Enabled := false;
      Timer_Snoopy_Hammer.Enabled := false;
      Timer_Snoopy_Hit_Note.Enabled := false;

      Inc(iScoreNextIntermission, 100);

      bControls := false;

      if (iScore >= iScoreNextChance) and (iMiss > 0) then                      // if score = "chance time" and there are misses
        begin                                                                     // special intermission (beeps repeatedly and cancels misses)
          Snd_Silence;

          Inc(iScoreNextChance, 1000); // sets next "chance" score

          iLoopIntermissionNoChance := 1;
          Timer_Intermission_No_Chance.Interval := 1;
          Timer_Intermission_No_Chance.Enabled := true;
        end
      else                                                                        // regular intermission
        begin
          if Odd(iHundreds) then
            Snd_Play(pfs_interm1)
          else
            Snd_Play(pfs_interm2);

          iLoopIntermission := 0;
          Timer_Intermission.Interval := 1;
          Timer_Intermission.Enabled := true;
        end;

      if (iScore >= iScoreNextChance) and (iMiss = 0) then                      // if score = "chance time" and there are misses
        begin                                                                     // activates chance time (score blinks and multiplier doubled)
          bChance := true;
          iScoreMultiplier := 2;
          Inc(iScoreNextChance, 1000); // sets next "chance" score

          iLoopScoreBlink := 1;
          Timer_Score_Blink.Interval := 450;
          Timer_Score_Blink.Enabled := true;
        end;
    end;
end;





procedure TForm1.Gam_Execute_Miss;
{ the misses counter is updated. If there are three misses, game ends. In no   }
{ such case, "chance time" ends - if running, and game reprises                }
begin
  Inc(iMiss); Vid_Show_Panel_Miss(iMiss);

  if iMiss < 3 then
    begin
      Timer_Score_Blink.Enabled := false;
      Vid_Show_Panel_Score(iScore, true);

      iScoreMultiplier := 1;
      bChance := false;

      Gam_Init_Stage;

      Vid_Show_All_Hero(false);
      iPosHeroX := 2; Vid_Show_Snoopy(iPosHeroX, 2, true);

      // game action starts
      bControls := true;
      bControlsHammer := true;

      iLoopWoodstock := 1;
      Timer_Woodstock.Interval := 300;
      Timer_Woodstock.Enabled := true;

      iLoopGameAnimation := 1;
      Timer_Game.Enabled := true;
    end
  else
    begin
      iLoopGameOver := 1;
      Timer_Game_Over.Interval := 1;
      Timer_Game_Over.Enabled := true;
    end;
end;





procedure TForm1.Gam_Init_Stage;
{ initialises screen and "disabled note" (on Game A only)                      }
  var
    iNote: integer;
begin
  Vid_Show_All_Notes(false);
  Vid_Show_All_Woodstock(false);

  Vid_Show_Woodstock_Row(2, true);

  im_lucy_1.Visible := true;
  im_piano_1.Visible := true;
  im_schroeder_1.Visible := true;

  // for "Game A" only: disables one note
  if (iMode = 6) then
    begin
      repeat
        iNote := Random(4) + 1;
      until
        iNote <> iNoteDisabledPosX;

      iNoteDisabledPosX := iNote;
    end;
end;





procedure TForm1.Gam_Set_Difficulty;
{ sets speed and other variables to make the game harder as score increases    }
  var
    iUnits,
    iHundreds: integer;
begin
  // calculates units and hundreds in score
  iUnits := iScore;
  iHundreds := 0;

  while iUnits > 99 do
    begin
      Dec(iUnits, 100);
      Inc(iHundreds);
    end;

  // speed (game and Snoopy's jumps)
  case iMode of
    6: begin                                                                    // Game A
         case iUnits of
            0..29: Timer_Game.Interval := 495;
           30..59: Timer_Game.Interval := 480;
           60..89: Timer_Game.Interval := 465;
           90..99: Timer_Game.Interval := 450;
         end; // case..of

         Timer_Game.Interval := Integer(Timer_Game.Interval) - (iHundreds * 15);

         if Timer_Game.Interval < 270 then
           Timer_Game.Interval := 270;

         Timer_Snoopy_Jump.Interval := Timer_Game.Interval + 90;
       end;
    8: begin                                                                    // Game B
         case iUnits of
            0..29: Timer_Game.Interval := 475;
           30..59: Timer_Game.Interval := 455;
           60..89: Timer_Game.Interval := 435;
           90..99: Timer_Game.Interval := 415;
         end; // case..of

         Timer_Game.Interval := Integer(Timer_Game.Interval) - (iHundreds * 20);

         if Timer_Game.Interval < 250 then
           Timer_Game.Interval := 250;

         Timer_Snoopy_Jump.Interval := Timer_Game.Interval + 80;
       end;
  end; // case..of

  // notes and random % for notes generation
  case iUnits of
    0..19: begin iNoteBaseTone := 0;  iMaxRandom := aDifficulty[iHundreds, 1]; end;
   20..39: begin iNoteBaseTone := 4;  iMaxRandom := aDifficulty[iHundreds, 2]; end;
   40..59: begin iNoteBaseTone := 8;  iMaxRandom := aDifficulty[iHundreds, 3]; end;
   60..69: begin iNoteBaseTone := 12; iMaxRandom := aDifficulty[iHundreds, 4]; end;
   70..79: begin iNoteBaseTone := 16; iMaxRandom := aDifficulty[iHundreds, 5]; end;
   80..89: begin iNoteBaseTone := 20; iMaxRandom := aDifficulty[iHundreds, 6]; end;
   90..99: begin iNoteBaseTone := 24; iMaxRandom := aDifficulty[iHundreds, 7]; end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // handheld is closed
         // does nothing
       end;
    1: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;
         bControlsHammer := false;
       end;
    2: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    3: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_game_background.Visible := true;
         aIm_Panel_Alarm[1].Visible := true;
         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bControls := false;
         bControlsHammer := false;
       end;
    4: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_game_background.Visible := true;
         aIm_Panel_Alarm[1].Visible := false;
         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bControls := false;
         bControlsHammer := false;
       end;
 5, 7: begin                                                                    // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         case iMode of
           5: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of

         bControls := false;
         bControlsHammer := false;
       end;
 6, 8: begin                                                                    // GAME A/B (Game A/B button release)
         im_game_background.Visible := true;

         iMiss := 0;
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         iScoreMultiplier := 1;
         iScoreNextIntermission := 100;
         iScoreNextChance := 300;
         bChance := false;

         Gam_Set_Difficulty; // includes "Timer_Game.Interval" setting

         Snd_Play(pfs_start);

         iLoopGameStart := 0;
         Timer_Game_Start.Interval := 1;
         Timer_Game_Start.Enabled := true;
       end;
    9: begin                                                                    // GAME OVER
         // does nothing
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_ClockTimer(Sender:TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j: integer;
    bGenerateNote,
    bPlaySound,
    bMiss: boolean;
begin
  // Lucy's and Schroeder's animation
  case iLoopGameAnimation of
    1: begin aIm_schroeder_hands[1].Visible := true;  aIm_schroeder_hands[2].Visible := true;  aIm_schroeder_hands[3].Visible := false; end;
    2: begin aIm_schroeder_hands[1].Visible := true;  aIm_schroeder_hands[2].Visible := false; aIm_schroeder_hands[3].Visible := true;  im_lucy_1_snore_1.Visible :=true; end;
    3: begin aIm_schroeder_hands[1].Visible := false; aIm_schroeder_hands[2].Visible := true;  aIm_schroeder_hands[3].Visible := true; end;
    4: begin aIm_schroeder_hands[1].Visible := true;  aIm_schroeder_hands[2].Visible := true;  aIm_schroeder_hands[3].Visible := false; im_lucy_1_snore_2.Visible :=true; end;
    5: begin aIm_schroeder_hands[1].Visible := true;  aIm_schroeder_hands[2].Visible := false; aIm_schroeder_hands[3].Visible := true; end;
    6: begin aIm_schroeder_hands[1].Visible := false; aIm_schroeder_hands[2].Visible := true;  aIm_schroeder_hands[3].Visible := true;  im_lucy_1_snore_1.Visible := false; im_lucy_1_snore_2.Visible := false; end;
  end; // case..of

  // loops action for animation
  Inc(iLoopGameAnimation);

  if iLoopGameAnimation = 7 then
    iLoopGameAnimation := 1;

  // notes movement and sound generation
  bPlaySound := false;

  for i := 5 downto 2 do
    begin
      aIm_notes[iLoopGame, i].Visible := aIm_notes[iLoopGame, i - 1].Visible;

      if aIm_notes[iLoopGame, i].Visible then
        bPlaySound := true;
    end;

  aIm_notes[iLoopGame, 1].Visible := false;

  // notes generation (genration comes automatically if no notes on screen)
  bGenerateNote := false;

  for i := 1 to 4 do
    for j := 1 to 4 do
      if aIm_notes[i, j].Visible then
        bGenerateNote := true;

  // notes generation (actual generation)
  if (iLoopGame <> iNoteDisabledPosX) // for Game A
    and ( (Random(100) >= iMaxRandom) or (not bGenerateNote) )
    and (not aIm_notes[iLoopGame, 2].Visible)
    and not ( (iLoopGame = 1) and (aIm_notes[4, 1].Visible) )then
    begin
      aIm_notes[iLoopGame, 1].Visible := true;
      bPlaySound := true;
    end;

  // miss check
  bMiss := false;

  for i := 1 to 4 do
    if aIm_notes[i, 5].Visible then
      begin
        iLoopMissNoteWoodstockPosX := i;
        bMiss := true;
      end;

  if bMiss then
    // execute miss procedure
    begin
      Gam_Stop_All_Timers;
      bControls := false;

      iLoopMissNoteWoodstock := 1;
      Timer_Miss_Note_Woodstock.Interval := 200;
      Timer_Miss_Note_Woodstock.Enabled := true;

      iLoopMissNoteSnoopy := 1;
      Timer_Miss_Note_Snoopy.Interval := 1;
      Timer_Miss_Note_Snoopy.Enabled := true;
    end
  else
    // action continues
    begin
      // loops action
      Inc(iLoopGame);

      if iLoopGame = 5 then
        iLoopGame := 1 ;

      // plays sound sample
      if bPlaySound then
        Snd_Play(aPFnote[iLoopGame + iNoteBaseTone]);
    end;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ "game over" music plays, then sets mode to "time", in 5 minutes time         }
begin
  case iLoopGameOver of
    1: begin                                                                    // plays "Game Over" tune
         Snd_Play(pfs_gameover);

         Inc(iLoopGameOver);
         Timer_Game_Over.Interval := 1700;
       end;
    2: begin                                                                    // sets the game as "Game Over"
         // game is over
         while iScore >= 1000 do
           Dec(iScore, 1000);

         if (iMode = 6) and (iScore > iHiScoreA) then begin iHiScoreA := iScore; iReg.WriteInteger('', 'HiScore1', iHiScoreA); end;
         if (iMode = 8) and (iScore > iHiScoreB) then begin iHiScoreB := iScore; iReg.WriteInteger('', 'HiScore2', iHiScoreB); end;

         iMode := 9;

         // Snoopy blinks, and Lucy snores for 5 minutes (variables settings)
         if bSnoopyJumpsLeft then
           iPosHeroX := 0
         else
           iPosHeroX := 5;

         if im_lucy_1_snore_2.Visible then
           iLoopGameOverBlink := 2
         else
           if im_lucy_1_snore_1.Visible then
             iLoopGameOverBlink := 1
           else
             iLoopGameOverBlink := 0;

         Inc(iLoopGameOver);
         Timer_Game_Over.Interval := 700;
       end;
    3: begin                                                                    // Snoopy blinks, and Lucy snores for 5 minutes (actual animation)
         case iLoopGameOverBlink mod 3 of
           0: im_lucy_1_snore_1.Visible := true;
           1: im_lucy_1_snore_2.Visible := true;
           2: begin im_lucy_1_snore_1.Visible := false; im_lucy_1_snore_2.Visible := false; end;
         end; // case..of

         aIm_snoopy[iPosHeroX].down.Visible := Odd(iLoopGameOverBlink);

         Inc(iLoopGameOverBlink);

         // sets the game to "Time" mode
         if iLoopGameOverBlink = 425 then
           begin
             iMode := 4; Gam_Set_Mode;

             Timer_Game_Over.Enabled := false;
           end;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ the starting tune plays and the game is set                                  }
begin
  case iLoopGameStart of
    0: begin Vid_Show_Woodstock_Row(2, true); im_piano_1.Visible := true; im_lucy_1.Visible := true;          im_schroeder_1.Visible := true;     aIm_schroeder_hands[1].Visible := true;  aIm_schroeder_hands[2].Visible := true;  Timer_Game_Start.Interval := 550; end;
    1: begin Vid_Show_Woodstock_Snore_Row(1, true);                       im_lucy_1_snore_1.Visible := true;                                                                               aIm_schroeder_hands[2].Visible := false; aIm_schroeder_hands[3].Visible := true;  Vid_Show_Notes_Row(1, true);  end;
    2: begin Vid_Show_Woodstock_Snore_Row(2, true);                                                           im_lucy_1_snore_2.Visible := true;  aIm_schroeder_hands[1].Visible := false; aIm_schroeder_hands[2].Visible := true;  aIm_schroeder_hands[3].Visible := false; Vid_Show_Notes_Row(1, false); Vid_Show_Notes_Row(2, true);  end;
    3: begin                                                              im_lucy_1_snore_1.Visible := false; im_lucy_1_snore_2.Visible := false; aIm_schroeder_hands[1].Visible := true;                                                                                    Vid_Show_Notes_Row(1, true);  Vid_Show_Notes_Row(2, false); Vid_Show_Notes_Row(3, true);  end;
    4: begin Vid_Show_Woodstock_Snore_Row(3, true);                       im_lucy_1_snore_1.Visible := true;                                                                               aIm_schroeder_hands[2].Visible := false; aIm_schroeder_hands[3].Visible := true;  Vid_Show_Notes_Row(1, false); Vid_Show_Notes_Row(2, true);  Vid_Show_Notes_Row(3, false); Vid_Show_Notes_Row(4, true);  end;
    5: begin
         Vid_Show_All_Woodstock(false);
         Vid_Show_Woodstock_Row(1, true);

         Vid_Show_Intermission_Frame(1);                                                                                                                       aIm_schroeder_hands[1].Visible := false; aIm_schroeder_hands[2].Visible := true;                                           Vid_Show_Notes_Row(1, true);  Vid_Show_Notes_Row(2, false); Vid_Show_Notes_Row(3, true);  Vid_Show_Notes_Row(4, false); Vid_Show_Notes_Row(5, true);

         Timer_Game_Start.Interval := 1000;
       end;
    6: begin
         Vid_Show_Woodstock_Row(1, false);
         Vid_Show_Woodstock_Row(2, true);

         Timer_Game_Start.Interval := 360;
       end;
    7: begin
         Vid_Show_Woodstock_Row(2, false);
         Vid_Show_Woodstock_Row(1, true);
       end;
    8: begin
         Vid_Show_Woodstock_Row(1, false);
         Vid_Show_Woodstock_Row(2, true);
       end;
    9: begin
         Vid_Show_Woodstock_Row(2, false);
         Vid_Show_Woodstock_Row(1, true);

         Vid_Show_Intermission_Frame(2);

         Timer_Game_Start.Interval := 140;
       end;
   10: begin
         Vid_Show_Woodstock_Row(1, false);
         Vid_Show_Woodstock_Row(2, true);

         Vid_Show_Intermission_Frame(3);

         Timer_Game_Start.Interval := 100;
       end;
   11: Vid_Show_Intermission_Frame(4);
   12: begin
         Vid_Show_Intermission_Frame(5);

         Timer_Game_Start.Interval := 1500; end;
   13: begin
         Vid_Show_Intermission_Frame(6);

         Gam_Init_Stage;

         Vid_Show_All_Hero(false);
         iPosHeroX := 2; Vid_Show_Snoopy(iPosHeroX, 2, true);

         // "disabled note" setting
         case iMode of
           6: begin
                iNoteDisabledPosX := 2;                                         // note column n. 2 is disabled at game start
                iLoopGame := 3;                                                 // start with the blue note
              end;
           8: begin
                iNoteDisabledPosX := 0;                                         // no note column is disabled
                iLoopGame := 2;                                                 // start with the green note
              end;
         end; // case..of

         // game action starts
         bControls := true;
         bControlsHammer := true;

         iLoopWoodstock := 1;
         Timer_Woodstock.Interval := 300;
         Timer_Woodstock.Enabled := true;

         iLoopGameAnimation := 1;
         Timer_Game.Enabled := true;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_IntermissionTimer(Sender: TObject);
{ this timer contains the implementation for the intermission that occurs at   }
{ every 100 points                                                             }
begin
  case iLoopIntermission of
    0: Timer_Intermission.Interval := 2400;
    1: Timer_Intermission.Interval := 2200;
    2: Timer_Intermission.Interval := 140;
    3: Timer_Intermission.Interval := 100;
    5: Timer_Intermission.Interval := 1500;
  end; // case..of

  Vid_Show_Intermission_Frame(iLoopIntermission);

  // loops action
  Inc(iLoopIntermission);

  if iLoopIntermission = 7 then
    begin
      Gam_Init_Stage;

      // game action starts
      bControls := true;
      bControlsHammer := true;

      iLoopWoodstock := 1;
      Timer_Woodstock.Interval := 300;
      Timer_Woodstock.Enabled := true;

      iLoopGameAnimation := 1;
      Timer_Game.Enabled := true;

      Timer_Snoopy_Hammer.Interval := 150;
      Timer_Snoopy_Hammer.Enabled := true; // brings the hammer back to "up"

      Timer_Intermission.Enabled := false;
    end;
end;





procedure TForm1.Timer_Intermission_No_ChanceTimer(Sender: TObject);
{ misses are reset to 0 but no "chance time" is awarded                        }
  var
    i: integer;
begin
  Snd_Play(pfs_chance);

  // misses counter blinks ten times
  if iMiss > 0 then
    begin
      i := 0; Vid_Show_Panel_Miss(i);
    end
  else
    Vid_Show_Panel_Miss(iMiss);

  // back to game (misses are set to 0, hammer is reset, note disappears)
  if iLoopIntermissionNoChance = 10 then
    begin
      iMiss := 0; Vid_Show_Panel_Miss(iMiss);

      bControls := true;

      Timer_Snoopy_Hammer.Interval := 150;
      Timer_Snoopy_Hammer.Enabled := true;

      iLoopSnoopyHitNote := 1;
      iLoopSnoopyHitNotePosX := iPosHeroX;
      Timer_Snoopy_Hit_Note.Interval := 1;
      Timer_Snoopy_Hit_Note.Enabled := true;

      Timer_Game.Enabled := true;
      Timer_Intermission_No_Chance.Enabled := false;
    end;

  Timer_Intermission_No_Chance.Interval := 270;

  // loops action
  Inc(iLoopIntermissionNoChance);
end;





procedure TForm1.Timer_Miss_FallTimer(Sender: TObject);
{ a miss has occured: Snoopy falls from the leftmost or rightmost platform     }
begin
  case iloopMissFall of
    1: begin
         Snd_Play(pfs_miss2);

         Timer_Miss_Fall.Interval := 400;
       end;
    2: begin
         if bSnoopyJumpsLeft then
           begin
             aIm_snoopy[0].up.Visible := false;
             aIm_snoopy[0].down.Visible := true;
           end
         else
           begin
             aIm_snoopy[4].up.Visible := false;
             aIm_snoopy[5].down.Visible := true;
           end;

         Timer_Miss_Fall.Interval := 600;
       end;
    3: if bSnoopyJumpsLeft then aIm_snoopy[0].down.Visible := false else aIm_snoopy[5].down.Visible := false;
    4: if bSnoopyJumpsLeft then aIm_snoopy[0].down.Visible := true  else aIm_snoopy[5].down.Visible := true;
    5: if bSnoopyJumpsLeft then aIm_snoopy[0].down.Visible := false else aIm_snoopy[5].down.Visible := false;
  end; // case..of

  Inc(iloopMissFall);

  if iloopMissFall = 7 then
    begin
      Gam_Execute_Miss;

      Timer_Miss_Fall.Enabled := false;
    end;
end;




procedure TForm1.Timer_Miss_Note_SnoopyTimer(Sender: TObject);
{ a miss has occured: a note has reached for Woodstock's nest. Snoopy jumps to }
{ the closest side of the screen, then falls from that platform. If he's       }
{ jumping when the miss occurs, he lands to the platform first.                }
{ Notice: the timer is not disabled here, but in the Timer_Miss_Note_Woodstock }
{ timer procedure                                                              }
begin
  case iLoopMissNoteSnoopy of
    1: begin
         if aIm_snoopy[2].up.Visible then
           if bSnoopyJumpsLeft then
             iLoopMissNoteSnoopy := 2
           else
             iLoopMissNoteSnoopy := 12;

           if aIm_snoopy[2].down.Visible then
             iLoopMissNoteSnoopy := 3;

           if aIm_snoopy[1].up.Visible then
             iLoopMissNoteSnoopy := 4;

           if aIm_snoopy[1].down.Visible then
             iLoopMissNoteSnoopy := 5;

           if aIm_snoopy[3].down.Visible then
             iLoopMissNoteSnoopy := 13;

           if aIm_snoopy[3].up.Visible then
             iLoopMissNoteSnoopy := 14;

           if aIm_snoopy[4].down.Visible then
             iLoopMissNoteSnoopy := 15;
       end;
    // to the left
    2: begin Vid_Show_Snoopy(2, 1, false);        Vid_Show_Snoopy(2, 2, true);        iLoopMissNoteSnoopy := 3; end;
    3: begin Vid_Show_Snoopy(2, 2, false);        Vid_Show_Snoopy(1, 1, true);        iLoopMissNoteSnoopy := 4; end;
    4: begin Vid_Show_Snoopy(1, 1, false);        Vid_Show_Snoopy(1, 2, true);        iLoopMissNoteSnoopy := 5; end;
    5: begin Vid_Show_Snoopy(1, 2, false);        aIm_snoopy[0].up.Visible := true;   iLoopMissNoteSnoopy := 6; end;
    6: begin aIm_snoopy[0].up.Visible := false;   aIm_snoopy[0].down.Visible := true; iLoopMissNoteSnoopy := 7; end;
    7: begin aIm_snoopy[0].down.Visible := false;                                     iLoopMissNoteSnoopy := 8; end; // blinks
    8: begin bSnoopyJumpsLeft := true;            aIm_snoopy[0].down.Visible := true; iLoopMissNoteSnoopy := 7; end; // blinks
    // to the right
   12: begin Vid_Show_Snoopy(2, 1, false);        Vid_Show_Snoopy(3, 2, true);        iLoopMissNoteSnoopy := 13; end;
   13: begin Vid_Show_Snoopy(3, 2, false);        Vid_Show_Snoopy(3, 1, true);        iLoopMissNoteSnoopy := 14; end;
   14: begin Vid_Show_Snoopy(3, 1, false);        Vid_Show_Snoopy(4, 2, true);        iLoopMissNoteSnoopy := 15; end;
   15: begin Vid_Show_Snoopy(4, 2, false);        Vid_Show_Snoopy(4, 1, true);        iLoopMissNoteSnoopy := 16; end;
   16: begin Vid_Show_Snoopy(4, 1, false);        aIm_snoopy[5].down.Visible := true; iLoopMissNoteSnoopy := 17; end;
   17: begin aIm_snoopy[5].down.Visible := false;                                     iLoopMissNoteSnoopy := 18; end; // blinks
   18: begin bSnoopyJumpsLeft := false;           aIm_snoopy[5].down.Visible := true; iLoopMissNoteSnoopy := 17; end; // blinks
  end; // case..of

  Timer_Miss_Note_Snoopy.Interval := 440;
end;





procedure TForm1.Timer_Miss_Note_WoodstockTimer(Sender: TObject);
{ a miss has occured: a note has reached for Woodstock's nest. This timer      }
{ contains the implementation for Woodstock's animation. This timer also       }
{ terminates the Timer_Miss_Note_Snoopy, managing Snoopy's blinking animation  }
begin
  case iLoopMissNoteWoodstock of
    1: begin
         Snd_Play(pfs_miss1);

         Vid_Show_Woodstock_Snore_Column(iLoopMissNoteWoodstockPosX, false);

         aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible := false;
         aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := true;

         Timer_Miss_Note_Woodstock.Interval := 485;
       end;
    2: begin aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := false;  aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible := true; end;
    3: begin aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible:= false; aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := true; end;
    4: begin aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := false;  aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible := true; end;
    5: begin aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible:= false; aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := true; end;
    6: begin aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := false;  aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible := true; end;
    7: begin aIm_woodstock[iLoopMissNoteWoodstockPosX].down.Visible:= false; aIm_woodstock[iLoopMissNoteWoodstockPosX].up.Visible := true;
         Timer_Miss_Note_Woodstock.Interval := 2100; end;
    8: begin
         Timer_Miss_Note_Snoopy.Enabled := false;

         Timer_Miss_Note_Woodstock.Interval := 500;
       end;
    9: begin
         Gam_Execute_Miss;

         Timer_Miss_Note_Woodstock.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMissNoteWoodstock);
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score blinks during chance time                                              }
begin
  Vid_Show_Panel_Score(iScore, not Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink >= 3 then
    iLoopScoreBlink := 1;
end;





procedure TForm1.Timer_Snoopy_HammerTimer(Sender: TObject);
{ Snoopy's hammer goes back to its original position (up)                      }
begin
 aIm_snoopy[iPosHeroX].hammer_down.Visible := false;
 aIm_snoopy[iPosHeroX].hammer_up.Visible := true;

  Timer_Snoopy_Hammer.Enabled := false;
end;





procedure TForm1.Timer_Snoopy_Hit_NoteTimer(Sender: TObject);
{ Snoopy hit a note. The note fades in two ticks                               }
  var
    i: integer;
begin
  case iLoopSnoopyHitNote of
    1: Timer_Snoopy_Hit_Note.Interval := 350;
    2: begin
         for i := 1 to 4 do
           aIm_notes_hit[i, 2].Visible := false;

         aIm_notes_hit[iLoopSnoopyHitNotePosX, 1].Visible := false;
         aIm_notes_hit[iLoopSnoopyHitNotePosX, 2].Visible := true;
       end;
    3: begin
         aIm_notes_hit[iLoopSnoopyHitNotePosX, 2].Visible := false;

         Timer_Snoopy_Hit_Note.Enabled := false;
       end;
  end; // case..of;

  // loops action
  Inc(iLoopSnoopyHitNote);
end;





procedure TForm1.Timer_Snoopy_JumpTimer(Sender: TObject);
{ Snoopy jumps left or right                                                   }
begin
  if bSnoopyJumpsLeft then
    begin                                                                       // jumps left
      Vid_Show_Snoopy(iPosHeroX - 1, 1, false);                                   // hides Snoopy
      Dec(iPosHeroX);                                                             // updates Snoopy's position
    end
  else
    begin                                                                       // jumps right
      Vid_Show_Snoopy(iPosHeroX, 1, false);                                       // hides Snoopy
      Inc(iPosHeroX);                                                             // updates Snoopy's position
    end;

  Vid_Show_Snoopy(iPosHeroX, 2, true);                                          // displays Snoopy in the new position

  bControls := true;                                                            // reactivates controller
  bControlsHammer := true;

  Timer_Snoopy_Jump.Enabled := false;
end;





procedure TForm1.Timer_WoodstockTimer(Sender: TObject);
{ Woodstock animation for the "snoring" sprites                                }
  var
    i: integer;
begin
  case iLoopWoodstock of
    1,
    2,
    3: Vid_Show_Woodstock_Snore_Row(iLoopWoodstock, true);
    4: for i := 1 to 3 do
         Vid_Show_Woodstock_Snore_Row(i, false);
  end;

  // loops action
  Inc(iLoopWoodstock);

  if iLoopWoodstock > 4 then
    iLoopWoodstock := 1;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
